package org.seasar.security.authorization;

import java.util.Arrays;
import java.util.Collections;

import org.aopalliance.intercept.MethodInvocation;
import org.jmock.Mock;
import org.jmock.MockObjectTestCase;
import org.seasar.security.authentication.AuthenticationContext;
import org.seasar.security.authentication.AuthenticationContextResolver;

/**
 * @author t-wada
 */
public class AbstractAuthorizationInterceptorMockTest extends MockObjectTestCase {
    private Mock mockAuthContext;
    private Mock mockInvocation;
    private Mock mockResolver;
    private AuthenticationContext authenticationContext;
    
    protected void setUp() throws Exception {
        mockInvocation = new Mock(MethodInvocation.class);
        mockInvocation.stubs().method("proceed");
        mockAuthContext = new Mock(AuthenticationContext.class);
        authenticationContext = (AuthenticationContext) mockAuthContext.proxy();
        mockResolver = new Mock(AuthenticationContextResolver.class);
        mockResolver.stubs().method("resolve").will(returnValue(authenticationContext));
    }
    private void verifyAll() {
        mockAuthContext.verify();
        mockInvocation.verify();
    }
    private void invokeTest(AbstractAuthorizationInterceptor toTest) throws Throwable {
        toTest.setAuthenticationContextResolver((AuthenticationContextResolver) mockResolver.proxy());
        toTest.invoke((MethodInvocation) mockInvocation.proxy());
        verifyAll();
    }
    
    AbstractAuthorizationInterceptor createInterceptor() {
        return new AbstractAuthorizationInterceptor() {
            protected Object ifInRole(MethodInvocation invocation, AuthenticationContext authContext, String roleName) throws Throwable {
                actualContext = authContext;
                actualRoleName = roleName;
                return null;
            }
            protected Object ifNotInRole(MethodInvocation invocation, AuthenticationContext authContext) throws Throwable {
                actualContext = authContext;
                return null;
            }
        };
    }
    private String actualRoleName;
    private AuthenticationContext actualContext;
    
    
    public void testEmptyRoleNameListCase() throws Throwable {
        mockAuthContext.expects(never()).method("isUserInRole");
        AbstractAuthorizationInterceptor toTest = createInterceptor();
        
        invokeTest(toTest);
        
        assertSame(authenticationContext, actualContext);
    }
    
    public void testStopIterationIfRoleMatches() throws Throwable {
        mockAuthContext.expects(atLeastOnce()).method("isUserInRole").withAnyArguments().will(onConsecutiveCalls(returnValue(false),returnValue(true)));
        AbstractAuthorizationInterceptor toTest = createInterceptor();
        toTest.addRoleName("roleD");
        toTest.addRoleName("roleE");
        toTest.addRoleName("roleF");
        
        invokeTest(toTest);
        
        assertEquals("roleE", actualRoleName);
        assertSame(authenticationContext, actualContext);
    }
    
    public void testNoRoleNameMatched() throws Throwable {
        mockAuthContext.expects(atLeastOnce()).method("isUserInRole").withAnyArguments().will(onConsecutiveCalls(returnValue(false),returnValue(false),returnValue(false)));
        AbstractAuthorizationInterceptor toTest = createInterceptor();
        toTest.addRoleName("roleD");
        toTest.addRoleName("roleE");
        toTest.addRoleName("roleF");
        
        invokeTest(toTest);
        
        assertSame(authenticationContext, actualContext);
    }
    
    public void testDoesNotAddDuplicatedRole() throws Throwable {
        AbstractAuthorizationInterceptor toTest = createInterceptor();
        toTest.addRoleName("roleF");
        toTest.addRoleName("roleG");
        toTest.addRoleName("roleF");
        
        assertEquals(Arrays.asList(new String[]{"roleF","roleG"}), toTest.getRoleNameList());
    }
    
    public void testDoesNotAddDuplicatedRoleNameList() throws Throwable {
        AbstractAuthorizationInterceptor toTest = createInterceptor();
        toTest.addRoleName("roleF");
        toTest.addRoleName("roleG");
        toTest.addRoleNames(Arrays.asList(new String[]{"roleG", "roleH", "roleF"}));
        
        assertEquals(Arrays.asList(new String[]{"roleF","roleG","roleH"}), toTest.getRoleNameList());
    }
    
    public void testIgnoreEmptyRoleListAddition() throws Exception {
        AbstractAuthorizationInterceptor toTest = createInterceptor();
        toTest.addRoleNames(Collections.EMPTY_LIST);
        
        assertEquals(0, toTest.getRoleNameList().size());
    }
}

