package org.seasar.security.authorization;

import java.util.List;

import org.aopalliance.intercept.MethodInvocation;
import org.jmock.Mock;
import org.jmock.MockObjectTestCase;
import org.seasar.security.authentication.AuthenticationContext;
import org.seasar.security.authentication.AuthenticationContextResolver;

/**
 * @author t-wada
 */
public class AllowInterceptorMockTest extends MockObjectTestCase {
    private Mock mockAuthContext;
    private Mock mockInvocation;
    private Mock mockResolver;
    
    protected void setUp() throws Exception {
        mockInvocation = mock(MethodInvocation.class);
        mockInvocation.stubs().method("proceed");
        mockAuthContext = mock(AuthenticationContext.class);
        mockResolver = mock(AuthenticationContextResolver.class);
    }
    
    private void denialCase(AllowInterceptor toTest) throws Throwable {
    	AuthenticationContext authContext = (AuthenticationContext) mockAuthContext.proxy();
    	mockResolver.stubs().method("resolve").will(returnValue(authContext));
        toTest.setAuthenticationContextResolver((AuthenticationContextResolver) mockResolver.proxy());
        Object deniedComponent = new Object();
        mockInvocation.expects(once()).method("getThis").will(returnValue(deniedComponent));
        try {
            // ## Act ##
            toTest.invoke((MethodInvocation) mockInvocation.proxy());
            
            // ## Assert ##
            fail("SAccessDeniedException should be thrown");
        } catch (SAccessDeniedException expected) {
            assertSame("contains target component", deniedComponent, expected.getTargetComponent());
            assertSame("contains authContext denied to access", authContext, expected.getDeniedAuthenticationContext());
        }
    }
    private void allowCase(AllowInterceptor toTest) throws Throwable {
        mockResolver.stubs().method("resolve").will(returnValue((AuthenticationContext) mockAuthContext.proxy()));
        toTest.setAuthenticationContextResolver((AuthenticationContextResolver) mockResolver.proxy());
        toTest.invoke((MethodInvocation) mockInvocation.proxy());
    }
    

    public void testAllowProceedIfUserIsInRole() throws Throwable {
        // ## Arrange ##
        mockAuthContext.expects(once()).method("isUserInRole").with(eq("roleA")).will(returnValue(true));
        AllowInterceptor toTest = new AllowInterceptor();
        toTest.addRoleName("roleA");
        
        allowCase(toTest);
    }
    
    public void testStoreDenialDetailsToSAccessDeniedException() throws Throwable {
        // ## Arrange ##
        mockAuthContext.expects(once()).method("isUserInRole").with(eq("roleB")).will(returnValue(false));
        AllowInterceptor toTest = new AllowInterceptor();
        toTest.addRoleName("roleB");
        
        denialCase(toTest);
    }
    
    public void testDoesNotThrowNullPointerIfNullIsPassedToListConstructor() throws Throwable {
        // ## Arrange ##
        mockAuthContext.expects(never()).method("isUserInRole");
        AllowInterceptor toTest = new AllowInterceptor((List)null);
        
        denialCase(toTest);
    }
    
    public void testDoesNotThrowExceptionIfNullIsPassedToStringConstructor() throws Throwable {
        // ## Arrange ##
        mockAuthContext.expects(never()).method("isUserInRole");
        AllowInterceptor toTest = new AllowInterceptor((String)null);
        
        denialCase(toTest);
    }
    
    public void testDenyAllAccessIfOnlyDefaultConstructorIsCalled() throws Throwable {
        // ## Arrange ##
        mockAuthContext.expects(never()).method("isUserInRole");
        AllowInterceptor toTest = new AllowInterceptor();
        
        denialCase(toTest);
    }
    
}
