package org.seasar.security.authorization;

import java.util.Arrays;
import java.util.List;

import org.aopalliance.intercept.MethodInvocation;
import org.jmock.Mock;
import org.jmock.MockObjectTestCase;
import org.seasar.security.authentication.AuthenticationContext;
import org.seasar.security.authentication.AuthenticationContextResolver;

/**
 * @author t-wada
 */
public class DenyInterceptorMockTest extends MockObjectTestCase {
    private Mock mockAuthContext;
    private Mock mockInvocation;
    private Mock mockResolver;
    
    protected void setUp() throws Exception {
        mockInvocation = mock(MethodInvocation.class);
        mockInvocation.stubs().method("proceed");
        mockAuthContext = mock(AuthenticationContext.class);
        mockResolver = mock(AuthenticationContextResolver.class);
    }
    
    private void denialCase(DenyInterceptor toTest, String expectedRole) throws Throwable {
        AuthenticationContext authContext = (AuthenticationContext) mockAuthContext.proxy();
        mockResolver.stubs().method("resolve").will(returnValue(authContext));
        toTest.setAuthenticationContextResolver((AuthenticationContextResolver) mockResolver.proxy());
        Object deniedComponent = new Object();
        mockInvocation.expects(once()).method("getThis").will(returnValue(deniedComponent));
        try {
            // ## Act ##
            toTest.invoke((MethodInvocation) mockInvocation.proxy());
            
            // ## Assert ##
            fail("SAccessDeniedException should be thrown");
        } catch (SAccessDeniedException expected) {
            assertSame("contains target component", deniedComponent, expected.getTargetComponent());
            assertSame("contains authContext denied to access", authContext, expected.getDeniedAuthenticationContext());
            assertEquals("containes denied roleName", expectedRole, expected.getDeniedRoleName());
        }
    }
    private void allowCase(DenyInterceptor toTest) throws Throwable {
        mockResolver.stubs().method("resolve").will(returnValue((AuthenticationContext) mockAuthContext.proxy()));
        toTest.setAuthenticationContextResolver((AuthenticationContextResolver) mockResolver.proxy());
        toTest.invoke((MethodInvocation) mockInvocation.proxy());
    }
    

    public void testAllowAccessIfRoleDoesNotMatch() throws Throwable {
        mockAuthContext.expects(once()).method("isUserInRole").with(eq("roleC")).will(returnValue(false));
        DenyInterceptor toTest = new DenyInterceptor("roleC");
        
        allowCase(toTest);
    }
    
    public void testStoreDenialDetailsToSAccessDeniedException() throws Throwable {
        mockAuthContext.expects(once()).method("isUserInRole").with(eq("roleB")).will(returnValue(true));
        DenyInterceptor toTest = new DenyInterceptor("roleB");
        
        denialCase(toTest, "roleB");
    }
    
    public void testPassDenyListToConstructor() throws Throwable {
        mockAuthContext.expects(atLeastOnce()).method("isUserInRole").withAnyArguments().will(onConsecutiveCalls(returnValue(false),returnValue(true)));
        DenyInterceptor toTest = new DenyInterceptor(Arrays.asList(new String[]{"roleB","roleC"}));
        
        denialCase(toTest, "roleC");
    }
    
    public void testDoesNotThrowNullPointerIfNullIsPassedToListConstructor() throws Throwable {
        mockAuthContext.expects(never()).method("isUserInRole");
        DenyInterceptor toTest = new DenyInterceptor((List)null);
        
        allowCase(toTest);
    }
    
    public void testDoesNotThrowNullPointerIfNullIsPassedToStringConstructor() throws Throwable {
        mockAuthContext.expects(never()).method("isUserInRole");
        DenyInterceptor toTest = new DenyInterceptor((String)null);
        
        allowCase(toTest);
    }
    
    public void testAllowAllAccessIfOnlyDefaultConstructorIsCalled() throws Throwable {
        mockAuthContext.expects(never()).method("isUserInRole");
        DenyInterceptor toTest = new DenyInterceptor();
        
        allowCase(toTest);
    }
    
}
